using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Security.Principal;
using System.Threading;

namespace ESS
{
    public partial class Form1 : Form
    {
        /// <summary>
        /// Thread used with the dataCollect() method to allow the GUI to continue running as code gets executed.
        /// </summary>
        private Thread tCollectData;

        public Form1()
        {
            InitializeComponent();

            WindowsIdentity currentUser = WindowsIdentity.GetCurrent();
            WindowsPrincipal currentPrincipal = new WindowsPrincipal(currentUser);
            if (!currentPrincipal.IsInRole(WindowsBuiltInRole.Administrator))
            {
                MessageBox.Show(@"You are not logged in as an administrator.
Right-click this program while holding shift and choose
'Run as...' to run using a privileged account.
You can still scan the local computer.");
            }
            CheckForIllegalCrossThreadCalls = false;

            this.textBox1.DragDrop += new System.Windows.Forms.DragEventHandler(this.textBox1_DragDrop);
            this.textBox1.DragEnter += new System.Windows.Forms.DragEventHandler(this.textBox1_DragEnter);
            this.textBox2.DragDrop += new System.Windows.Forms.DragEventHandler(this.textBox2_DragDrop);
            this.textBox2.DragEnter += new System.Windows.Forms.DragEventHandler(this.textBox2_DragEnter);

            checkBox1.Text = "Use account other than " + currentPrincipal.Identity.Name;
            toolTip1.SetToolTip(textBox3,
                "You may enter an IPv4 address, a hostname, or a fully-qualified domain name (FQDN).");
            toolTip2.SetToolTip(textBox1,
                "Input file should be a plaintext file with one hostname, FQDN, or IP per line.");
            toolTip3.SetToolTip(textBox6,
                "Username should be of the form DOMAIN\\Username.");
            toolTip4.SetToolTip(textBox7,
                "The password cannot be blank. If the remote account has a blank password you " +
                "will not be able to use that account remotely.");
        }

        private void button1_Click(object sender, EventArgs e)
        {
            openFileDialog1.ShowDialog();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            saveFileDialog1.ShowDialog();
        }

        private void openFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            textBox1.Text = openFileDialog1.FileName;
        }

        private void button5_Click(object sender, EventArgs e)
        {
            try
            {
                tCollectData = new Thread(new ThreadStart(dataCollect));
                tCollectData.Start();
            }
            catch (Exception oExcept)
            {
                MessageBox.Show(oExcept.ToString());
                try
                {
                    tCollectData.Abort();
                }
                catch (Exception)
                {
                    // ignore the ThreadStateException.
                }
            }
            //dataCollect();
        }

        private void button4_Click(object sender, EventArgs e)
        {
            try
            {
                if (tCollectData.IsAlive)
                {
                    tCollectData.Abort();
                }
            }
            catch (Exception)
            {
                // I know there will be a ThreadStateException...ignore it and quit already.
            }
            Close();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            AboutBox1 myAbout = new AboutBox1();
            myAbout.Show();
        }

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            textBox2.Text = saveFileDialog1.FileName;
        }

        /// <summary>
        /// This method will read input out of the input fields, create an EagleSpreadSheet object and
        /// execute the entire thing. This method should be called asynchronously, although for debugging
        /// you can remove the threading code in button4_Click().
        /// Provides a little bit of data/input validation, but serious internal errors are usually thrown
        /// by EagleSpreadSheet() and caught here, then shown to the user in a MessageBox without formatting.
        /// </summary>
        /// <remarks>
        /// If an error occurs it is better to close the program than to call dataCollect() again.
        /// </remarks>
        private void dataCollect()
        {
            if (string.IsNullOrEmpty(textBox2.Text))
            {
                MessageBox.Show("No output file specified.");
                return;
            }

            if (checkBox1.Checked && string.IsNullOrEmpty(textBox6.Text))
            {
                MessageBox.Show("Username cannot be blank.");
                return;
            }

            if (checkBox1.Checked && string.IsNullOrEmpty(textBox7.Text))
            {
                MessageBox.Show("Password cannot be blank.");
                return;
            }

            if (radioButton2.Checked && string.IsNullOrEmpty(textBox1.Text))
            {
                MessageBox.Show("Input file not specified.");
                return;
            }

            button5.Enabled = false;

            bool use_csv;
            if (textBox2.Text.EndsWith("csv"))
                use_csv = true;
            else
                use_csv = false;

            EagleSpreadSheet myess;

            /* All these different possible input configurations are the reason EagleSpreadSheet() has so
             * many overloaded constructors. Read the documentation for whatever one you use. There shouldn't
             * be any side effects, but the usage of each one varies with the data types you can provide. */
            if (radioButton1.Checked)
            {
                myess = new EagleSpreadSheet(textBox2.Text);
            }
            else if (radioButton2.Checked)
            {
                myess = new EagleSpreadSheet(textBox2.Text, textBox1.Text);
            }
            else if (radioButton3.Checked)
            {
                try
                {
                    System.Net.IPAddress myIP = System.Net.IPAddress.Parse(textBox3.Text);
                    myess = new EagleSpreadSheet(textBox2.Text, myIP, textBox3.Text);
                }
                catch (FormatException)
                {
                    myess = new EagleSpreadSheet(textBox2.Text, null, textBox3.Text);
                }
                catch (Exception e)
                {
                    MessageBox.Show(e.ToString());
                    return;
                }
            }
            else if (radioButton4.Checked)
            {
                System.Net.IPAddress startIP, endIP;
                try
                {
                    startIP = System.Net.IPAddress.Parse(textBox4.Text);
                }
                catch (FormatException)
                {
                    MessageBox.Show(textBox4.Text + " is not a valid IP address.");
                    return;
                }
                try
                {
                    endIP = System.Net.IPAddress.Parse(textBox5.Text);
                }
                catch (FormatException)
                {
                    MessageBox.Show(textBox5.Text + " is not a valid IP address.");
                    return;
                }
                myess = new EagleSpreadSheet(textBox2.Text, startIP, endIP);
            }
            else
            {
                return;
            }


            myess.Output_Format_CSV = use_csv;
            myess.external_progress_bar = progressBar1;
            myess.run_status_label = label3;

            myess.DefineColumns(checkBoxHostname.Checked, checkBoxDomain.Checked,
                checkBoxIP.Checked, checkBoxMAC.Checked, checkBoxOS.Checked, checkBoxCPU.Checked,
                checkBoxHDD.Checked, checkBoxRAM.Checked, checkBoxMake.Checked, checkBoxModel.Checked,
                checkBoxSerial.Checked, checkBoxWarranty.Checked);

            if (checkBox1.Checked)
            {
                myess.userName = textBox6.Text;
                myess.userPass = textBox7.Text;
            }

            progressBar1.Value = 0;
            try
            {
                myess.MasterDataCollect();
                label3.Text = "Data collection complete.";
                progressBar1.Value = 100;
                MessageBox.Show("Data collection complete.");
            }
            catch (Exception e)
            {
                label3.Text = "An error has occurred.";
                MessageBox.Show(e.ToString());
                return;
            }
            finally
            {
                myess.fout.Close();
                label3.Text = "When ready to collect data click Run. Data collection may take a long time.";
                progressBar1.Value = 0;
                button5.Enabled = true;
            }
        }

        private void textBox1_DragEnter(object sender, System.Windows.Forms.DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
                e.Effect = DragDropEffects.All;
            else
                e.Effect = DragDropEffects.None;
        }

        private void textBox1_DragDrop(object sender, System.Windows.Forms.DragEventArgs e)
        {
            string[] saDrop = (string[])e.Data.GetData(DataFormats.FileDrop, false);
            if (saDrop.Length == 1)
            {
                textBox1.Text = saDrop[0];
            }
        }

        private void textBox2_DragEnter(object sender, System.Windows.Forms.DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
                e.Effect = DragDropEffects.All;
            else
                e.Effect = DragDropEffects.None;
        }

        private void textBox2_DragDrop(object sender, System.Windows.Forms.DragEventArgs e)
        {
            string[] saDrop = (string[])e.Data.GetData(DataFormats.FileDrop, false);
            if (saDrop.Length == 1)
            {
                textBox2.Text = saDrop[0];
            }
        }

        private void radioButton3_CheckedChanged(object sender, EventArgs e)
        {
            textBox3.Enabled = true;
            textBox4.Enabled = false;
            label4.Enabled = false;
            textBox5.Enabled = false;
            textBox1.Enabled = false;
            button1.Enabled = false;
        }

        private void radioButton4_CheckedChanged(object sender, EventArgs e)
        {
            textBox3.Enabled = false;
            textBox4.Enabled = true;
            label4.Enabled = true;
            textBox5.Enabled = true;
            textBox1.Enabled = false;
            button1.Enabled = false;
        }

        private void radioButton2_CheckedChanged(object sender, EventArgs e)
        {
            textBox3.Enabled = false;
            textBox4.Enabled = false;
            label4.Enabled = false;
            textBox5.Enabled = false;
            textBox1.Enabled = true;
            button1.Enabled = true;
        }

        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {
            textBox3.Enabled = false;
            textBox4.Enabled = false;
            label4.Enabled = false;
            textBox5.Enabled = false;
            textBox1.Enabled = false;
            button1.Enabled = false;
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox1.Checked)
            {
                label1.Enabled = true;
                textBox6.Enabled = true;
                label5.Enabled = true;
                textBox7.Enabled = true;
            }
            else
            {
                label1.Enabled = false;
                textBox6.Enabled = false;
                label5.Enabled = false;
                textBox7.Enabled = false;
            }
        }

        private void textBox2_TextChanged(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBox2.Text))
            {
                button5.Enabled = false;
            }
            else
            {
                button5.Enabled = true;
            }
        }
    }
}